﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;

namespace WpfInfragisticsTest
{
    public static class PwdHelper
    {
        public static readonly DependencyProperty Binding =
          DependencyProperty.RegisterAttached("Binding", typeof(string), typeof(PwdHelper), new PropertyMetadata(string.Empty, OnBindingChanged));

        public static readonly DependencyProperty Bind = DependencyProperty.RegisterAttached(
            "Bind", typeof(bool), typeof(PwdHelper), new PropertyMetadata(false, OnBindChanged));

        private static readonly DependencyProperty UpdatingPassword =
            DependencyProperty.RegisterAttached("UpdatingPassword", typeof(bool), typeof(PwdHelper), new PropertyMetadata(false));

        private static void OnBindingChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            PasswordBox box = d as PasswordBox;

            // only handle this event when the property is attached to a PasswordBox
            // and when the Bind attached property has been set to true
            if (d == null || !GetBind(d))
            {
                return;
            }

            // avoid recursive updating by ignoring the box's changed event
            box.PasswordChanged -= HandlePasswordChanged;

            string newPassword = (string)e.NewValue;

            if (!GetUpdatingPassword(box))
            {
                box.Password = newPassword;
            }

            box.PasswordChanged += HandlePasswordChanged;
        }

        private static void OnBindChanged(DependencyObject dp, DependencyPropertyChangedEventArgs e)
        {
            // when the Bind attached property is set on a PasswordBox,
            // start listening to its PasswordChanged event

            PasswordBox box = dp as PasswordBox;

            if (box == null)
            {
                return;
            }

            bool wasBound = (bool)(e.OldValue);
            bool needToBind = (bool)(e.NewValue);

            if (wasBound)
            {
                box.PasswordChanged -= HandlePasswordChanged;
            }

			if (needToBind && !GetUpdatingPassword(box))
			{
				box.Password = GetBinding(dp);
			}

            if (needToBind)
            {
                box.PasswordChanged += HandlePasswordChanged;
            }
        }

        private static void HandlePasswordChanged(object sender, RoutedEventArgs e)
        {
            PasswordBox box = sender as PasswordBox;

            // set a flag to indicate that we're updating the password
            SetUpdatingPassword(box, true);
            // push the new password into the Binding property
            SetBinding(box, box.Password);
            SetUpdatingPassword(box, false);
        }

        public static void SetBind(DependencyObject dp, bool value)
        {
            dp.SetValue(Bind, value);
        }

        public static bool GetBind(DependencyObject dp)
        {
            return (bool)dp.GetValue(Bind);
        }

        public static string GetBinding(DependencyObject dp)
        {
            return (string)dp.GetValue(Binding);
        }

        public static void SetBinding(DependencyObject dp, string value)
        {
            dp.SetValue(Binding, value);
        }

        private static bool GetUpdatingPassword(DependencyObject dp)
        {
            return (bool)dp.GetValue(UpdatingPassword);
        }

        private static void SetUpdatingPassword(DependencyObject dp, bool value)
        {
            dp.SetValue(UpdatingPassword, value);
        }
    }
}
